#!/usr/bin/perl -w
#
# common.pl
#
# Common routines shared by several of the WebFront scripts.
# This file contains those scripts not related to HTML output.
#
# Copyright 1998 Corel Computer Corporation
#
# This is open-source software (see www.opensource.org).  Open Source(R)
# is a Registered Certification Mark of Software in the Public Interest.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 1,
# or, (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to the
# Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;

# Determine the fully-qualified hostname of the system

sub hostname()
{
  my $hostname = `/bin/hostname -f`;
  $hostname =~ s/[ \n\r]//g;
  return $hostname;
}

# Determine the ip-address of the current host

sub hostip()
{
  my $eth = `cat /etc/sysconfig/network|grep GATEWAYDEV|cut -d= -f2`;
  $eth =~ s/[ \n\r]//g;
  my $ip = `/sbin/ifconfig $eth|grep inet|cut -d: -f2|cut -d' ' -f1`;
  $ip =~ s/[ \n\r]//g;
  return $ip;
}

# This one is a convenience function, it can be redefined as needed
# so that links on any CGI-scripted pages can be set to either a
# numeric or a symbolic address.  Note you need DNS for the latter.

sub host()
{
  return hostname();
}

# Determine the name of user web publishing directories
# (by scanning through the Apache configuration file)

sub publish_dir_name()
{
  my $dirname = `grep ^UserDir /etc/httpd/conf/srm.conf|cut -d' ' -f2`;
  chomp $dirname;
  return $dirname;
}

# Reading the password file into a hash-of-lists

sub read_passwd() 
{
  my %users;			# Results will be stored in here
  open FH,"/etc/passwd" or die;	# Open password file for reading
  while (<FH>)			# Read each line from the file
  {
    chomp;			# Get rid of CR/LF at end of line
    my @entry = split /:/;	# Split into fields at ":" separators
    my $name = $entry[0];	# Pull out the users' login name
    $users{$name} = [@entry]	# Add entire entry to hash-of-lists
    unless $entry[2] < 101;	# except if the UID is less than 100
  }
  close FH;
  return %users;
}

# Read the group list into a hash-of-lists

sub read_group()
{
  my %groups;			# Placeholder for the result
  my $userfield = 0;		# 
  open FH,"/etc/group" or die;	# Open group file for reading
  while (<FH>)			# Read each line in turn
  {
    chomp;			# Strip of trailing CR/LF's
    my @entry = split /:/;	# Split at ":" boundaries
    my $name = $entry[0];	# Pull out the group name
    $groups{$name} = [@entry]	# Push onto hash-of-lists
    unless $entry[2] < 100;	# except if the GID is less than 100
  }
  close FH;

  #Deal with users with gid set but no entry in /etc/groups

  my %users = read_passwd() ;
  my @userkey = keys %users ;
  my @groupkey = keys %groups ;
  for ( my $eachuser = 0 ; $eachuser <= $#userkey ; $eachuser++ )
  {
    my $userinfo = $users{$userkey[$eachuser]};
    for ( my $eachgroup = 0 ; $eachgroup <= $#groupkey ; $eachgroup++ )
    {
      my $groupinfo = $groups{$groupkey[$eachgroup]};

      if ( $$groupinfo[2] eq $$userinfo[3] )  # if gids match
      {
        my $exist = 0;
        my @groupmembers = split(/,/, $$groupinfo[3]);
        for ( my $eachmember = 0 ; $eachmember <= $#groupmembers ; $eachmember++ )
        {
          if ( $groupmembers[$eachmember] eq $$userinfo[0] ) # if user is in groupfile
          {
            $exist = 1;
            last;
          }
        }
        if ( $exist ne 1 )
        {
          $groupmembers[$#groupmembers+1] = $$userinfo[0];
          $$groupinfo[3] = join(',', @groupmembers);
          $groups{$groupkey[$eachgroup]} = $groupinfo;
        }
      }
    }
  }
  return %groups
}

# Find out which groups user is a member of.  ARGS: username

sub users_group
{
  my %groups = read_group();
  my @usersgroups = ();
  my $counter = 0 ;
  my @groupkey = keys %groups;
  for ( my $eachgroup = 0 ; $eachgroup <= $#groupkey ; $eachgroup++ )
  {
    my $groupinfo = $groups{$groupkey[$eachgroup]};
    my @groupmembers = split(/,/, $$groupinfo[3]);
    for ( my $eachmember = 0 ; $eachmember <= $#groupkey ; $eachmember++)
    {
      if ( $_[0] eq $groupmembers[$eachmember] ) # if user is a member
      {
        $usersgroups[$counter] = $$groupinfo[0]; $counter++;
      }
    }
  }
  return @usersgroups;
}

#Determine is user belongs to a group. Return 1 for true, 0 for false. ARGS: user, group

sub is_member
{
  my %groups = read_group();
  my $groupinfo = $groups{$_[1]};
  my @groupmembers = (defined $$groupinfo[3]) ? 
			split(/,/, $$groupinfo[3]) : ();
  for ( my $eachmember = 0 ; $eachmember <= $#groupmembers ; $eachmember++)
  {
    if ( $_[0] eq $groupmembers[$eachmember] ) # if user is a member
    {
      return 1;
    }
  }
  return 0; 
}

sub find_free_uid
{
  my %users = read_passwd();
  my @key = keys %users ;
  my @uids = ();
  for ( my $eachuser ; $eachuser <= $#key ; $eachuser++ )
  {
    my $currentuser = $users{ $key [$eachuser] } ;
    $uids[$eachuser] = $$currentuser[2];
  }
  my @sorted_uids = ( sort by_number @uids );
  for ( my $eachuid ; $eachuid <= $#sorted_uids ; $eachuid++ )
  {
    if ((( $sorted_uids[$eachuid] + 1 ) ne $sorted_uids[( $eachuid +1 )] ) && ( $sorted_uids[$eachuid] ne ($sorted_uids[$eachuid + 1] )) && ( $sorted_uids[$eachuid] >= 500 )) 
    {
      # Found a free UID, now return it 
      return ( $sorted_uids[$eachuid] + 1 ); 
    }
  }
  # No reclaimable UIDs found.  Return next highest.
  return ( $sorted_uids[$#sorted_uids] + 1 );
}


sub find_free_gid
{
  my %groups = read_group();
  my @key = keys %groups ;
  my @gids = ();
  for ( my $eachgroup ; $eachgroup <= $#key ; $eachgroup++ )
  {
    my $currentgroup = $groups{ $key [$eachgroup] } ;
    $gids[$eachgroup] = $$currentgroup[2];
  }
  my @sorted_gids = ( sort by_number @gids );
  for ( my $eachgid ; $eachgid <= $#sorted_gids ; $eachgid++ )
  {
    if ((( $sorted_gids[$eachgid] + 1 ) ne $sorted_gids[( $eachgid +1 )] ) && ( $sorted_gids[$eachgid] ne ($sorted_gids[$eachgid + 1] )) && ( $sorted_gids[$eachgid] >= 100 )) 
    {
      # Found a free GID, now return it 
      return ( $sorted_gids[$eachgid] + 1 ); 
    }
  }
  # No reclaimable UIDs found.  Return next highest.
  return ( $sorted_gids[$#sorted_gids] + 1 );
}

sub by_number
{
  if ( $a < $b ) { return -1; }
  elsif ( $a eq $b ) { return 0; }
  elsif ( $a > $b ) { return 1; }
}


# Read /etc/shells into an array

sub read_shells()
{
  my @shells;                                 # Results will be stored in here
  open FH,"/etc/shells" or die;               # Open file for reading
  @shells = <FH> ;                            # Read in the file
  for ( my $i = 0 ; $i <= $#shells ; $i++ )   
  {
    chomp $shells[$i] ;
  }
  return @shells;
}

# Return the name of a group

# Return TRUE so this module can be loaded
1;
